EXTERN rotate_byte, rotate_mirror, dirty_rects

PUBLIC put_sprite, put_mirror, delete_sprite, mask_sprite, spritedata

MACRO pixeladdressdown
LOCAL done
                inc h
                ld a,h
                and 7
                jp nz,done
                ld a,l
                add a,32
                ld l,a
                jp c,done
                ld a,h
                sub 8
                ld h,a
done:
ENDM

MACRO markdirty value
  ; b = y, c = x
                ld a,b
                and %11111000
                ld h,0
                ld l,a
                add hl,hl           ; y / 8 x 32
                add hl,hl
                ld a,c
                rra
                rra
                rra
                and %00011111
                add l
                ld l,a
                ld de,dirty_rects
                add hl,de
                ld a,value
                ld bc,30
                ld (hl),a         ; put value in buffer to "delete" or "clear" block
                inc hl
                ld (hl),a
                inc hl
                ld (hl),a
                add hl,bc
                ld (hl),a         
                inc hl
                ld (hl),a
                inc hl
                ld (hl),a
                add hl,bc

                ld (hl),a
                inc hl
                ld (hl),a
                inc hl
                ld (hl),a
                
                ret                
ENDM

MACRO put_sprite_macro lookup, first, second    ; normal or mirror, byte order from pop data ixl/ixh
LOCAL setsp,loop,restoresp,onboundry,obloop

                ld (restoresp+1),sp
                ld (setsp+1),hl
                call getpixeladdress    ; screen address in hl, a = rotateoffset

                ld ixl,a                ; save offset
                ld de,spritemasks
                add e
                ld e,a                  ; add offset
                ld a,(de)               ; load rotation mask
                ld b,a                  ; mask in b
                cpl
                ld c,a                  ; inverse mask in c
                ld a,ixl                ; restore offset in a
                add lookup/256          ; a = lookup addr high
                ld d,a

                ld iyh,8               ; 16 row sprite
                di
setsp:          ld sp,0

                dec ixl
                jp c,onboundry

loop:
                pop ix                  ; get sprite data
                ld e,first              ; de = rotation
                ld a,(de)               ; rotated data
                and b                   ; mask
                or (hl)                 ; scren data
                ld (hl),a               ; put
                inc l
                ld a,(de)               ; rotated data
                and c                   ; inverse mask
                or (hl)                 ; scren data
                ld (hl),a               ; put

                ld e,second
                ld a,(de)               ; rotated data
                and b                   ; mask
                or (hl)                 ; screen data
                ld (hl),a               ; put
                inc l
                ld a,(de)               ; rotated data
                and c                   ; inverse mask
                or (hl)                 ; scren data
                ld (hl),a

                pixeladdressdown        ; backwards odd rows

                pop ix
                ld e,second
                ld a,(de)               ; rotated data
                and c                   ; inverse mask
                or (hl)                 ; scren data
                ld (hl),a
                dec l
                ld a,(de)               ; rotated data
                and b                   ; mask
                or (hl)                 ; screen data
                ld (hl),a               ; put

                ld e,first              ; de = rotation
                ld a,(de)               ; rotated data
                and c                   ; inverse mask
                or (hl)                 ; scren data
                ld (hl),a               ; put
                dec l
                ld a,(de)               ; rotated data
                and b                   ; mask
                or (hl)                 ; scren data
                ld (hl),a               ; put

                pixeladdressdown
                dec iyh
                jp nz,loop

restoresp:
                ld sp,0
                ei
                ret

onboundry:                      ; including lookup for the mirror case, case of on boundry and not mirrored not optimized
                ld b,8
obloop:         pop ix
                ld e,first
                ld a,(de)
                or (hl)
                ld (hl),a
                inc l
                ld e,second
                ld a,(de)
                or (hl)
                ld (hl),a
                pixeladdressdown
                pop ix
                ld e,second
                ld a,(de)
                or (hl)
                ld (hl),a
                dec l
                ld e,first
                ld a,(de)
                or (hl)
                ld (hl),a
                pixeladdressdown

                djnz obloop

                jp restoresp

ENDM

put_sprite:
                put_sprite_macro rotate_byte, ixl, ixh

put_mirror:
                put_sprite_macro rotate_mirror, ixh, ixl

delete_sprite:
                markdirty $c0
mask_sprite:
                markdirty $c9

getpixeladdress:                  ; b = y, c = x
                ld a,b
                and %00000111
                or %01000000
                ld h,a
                ld a,b
                rra
                rra
                rra
                and %00011000
                or h
                ld h,a
                ld a,b
                rla
                rla
                and %11100000
                ld l,a
                ld a,c
                and %11111000
                rra
                rra
                rra
                or l
                ld l,a
                ld a,c
                and 7               ; return address in hl, x mod 8 in a
                ret

spritemasks:
                db %11111111
                db %01111111
                db %00111111
                db %00011111
                db %00001111
                db %00000111
                db %00000011
                db %00000001

spritedata:     db @"--------",@"--------"
                db @"----####",@"####----"
                db @"---##-##",@"--------"
                db @"--##-##-",@"#####---"
                db @"--##-##-",@"#####---"
                db @"--#####-",@"#####---"
                db @"--######",@"--------"
                db @"---#####",@"######--"
                db @"-----###",@"#####---"
                db @"------##",@"##------"
                db @"----####",@"########"
                db @"----####",@"---###--"
                db @"------##",@"##------"
                db @"----###-",@"###-----"
                db @"----###-",@"-###----"
                db @"----####",@"-####---"
